;*********************************************************************************************************
;		Zyklengenaue Emulation eines MOS6510 auf einem ATmega
;			(c) Mark Leyer 2010
;*********************************************************************************************************







;*********************************************************************************************************








;*********************************************************************************************************
;	Variable Festlegungen fr den Emulator:

.include "m644def.inc"


;.equ Vector6510RST	 	= 0x0FFE*2			; Startadresse fr den 6510
;.equ Vector6510IRQ		= 0x0FFF*2
;.equ Vector6510NMI		= 0x0FFD*2
.equ Vector6510NMI		= 0xFFFA
.equ Vector6510RST		= 0xFFFC
.equ Vector6510IRQ		= 0xFFFE

; Port B fr RDY, NMI und IRQ:
; IRQ auf INT2/PB2
; NMI auf PB1
; RDY auf PB0
;	je nach Anwendung noch ndern!!!!!
; Port C als 6510-IO-Port
;.equ ADDR_L_port = PORTA
;.equ ADDR_H_port = PORTB
;.equ DATA_port = PINC
;.equ CE_port = PORTC

;Vorsicht mit PortC, am 644 sind PC2-PC5 vermutlich kaputt

.equ SYS_port = PINC		; fr IRQ, NMI und RDY
.equ CPU_DDR = DDRD			; 6502-Adresse $0000
.equ CPU_I = PIND			; 6502-Adresse $0001
.equ CPU_O = PORTD			; 6502-Adresse $0001
; Port fr NMI ist derzeit nicht aktiviert
;		=> NMI ber interne AVR-IRQ
.equ NMI_bit = 0x01
.equ IRQ_bit = 0x02
.equ RDY_bit = 0x00
.equ IRQMask = EIMSK
.equ IRQMPin = INT2

.equ ZPStart = 0x0F			; Highbyte der ZP zu Beginn

.equ ROMBASE = 0x18
.equ IOBASE = 0x17

.equ MHZ = 30				; ganzzahlige MHz fr die Warteschleifen
							; knnte helfen die Taktung besser hinzubekommen





.undef XL
.undef XH


; noch frei:
;				r0	r1	r2	r3	r11
;				
;	evtl. weg	r4	r5	r9	r10


.def opcode = r20
;.def opaddr = r21
.def opbyte2 = r10
.def opbyte3 = r9
.def oparand = r8
.def ADD_L = r7
.def ADD_H = r6
.def ADC_helper1 = r5
.def ADC_helper2 = r4
.def ZP_Highbyte = r3

.def temp = r16
.def temp2 = r17
.def temp3 = r23
.def loop = r19
.def zero = r13

.def reg_A = r18
.def reg_X = r14
.def reg_Y = r15
.def reg_PCL = r26
.def reg_PCH = r27
.def reg_SP = r12
.def reg_Status = r22			;
.def reg_Status2 = r21
.def SyncL = r24
.def SyncH = r25
.def SyncCount = r11
;						 7	6	5	4	3	2	1	0
;						 N	V	-	B	D	I	Z	C

;		Status2:		I	T	H	S	-	-	-	-	ORIG
;						-	-	-	B	D	I	-	-
;		Status:			-	-	-	-	V	N	Z	C


.equ flag_N = 0b00000100
.equ flag_V = 0b00001000
.equ flag_Z = 0b00000010
.equ flag_C = 0b00000001
.equ flag_I = 0b00000100
.equ flag_D = 0b00001000
.equ flag_B = 0b00010000
.equ flagN = 0x02
.equ flagV = 0x03
.equ flagZ = 0x01
.equ flagC = 0x00
.equ flagI = 0x02
.equ flagD = 0x03
.equ flagB = 0x04




; keine Ahnung, wie ich den RAM anschliee und anspreche, daher hier eigentlich nur Dummy
.macro READ_EXT_RAM
			sbic SYS_PORT, RDY_bit		; Leitung frei??
			rcall Warteschleife
			out ADDR_L_port, @0
			out ADDR_H_port, @1
			;out CE_port, 0b00000011		; RE oder so setzen
			nop; freihalten zur Sicherheit
			in @2, DATA_port

		; Variante internes SRAM:
		;	sbic IRQ_PORT, RDY_bit
		;	rcall Warteschleife
		;	mov YL, @0
		;	mov YH, @1
		;	ld @2, Y

.endmacro
;Gesamt 6 Takte!!


.macro READ_RAMROM
			cpi ZH, ROMBASE	; test, ob RAM oder ROM
			brsh PC+3
			ld @0, Z+	
			rjmp PC+2	;6
		
			lpm @0, Z+	;6
.endmacro

.macro addSync1
			adiw SyncL, @0
			;adc SyncH, zero
.endmacro
.macro addSync2
			ldi loop, @0
			add SyncL, loop
			adc SyncH, zero
.endmacro



.macro READ_RDY
			sbic SYS_PORT, RDY_bit		; Leitung frei??
			rcall Warteschleife
.endmacro
;Gesamt 2 Takte!!

.macro WRITE_EXT_RAM
			out ADDR_L_port, @0
			out ADDR_H_port, @1
			nop
			out DATA_port, @2
.endmacro

.macro LoadZ
			ldi ZL, low (@0)
			ldi ZH, high(@0)
.endmacro



.include "WAIT_macro_inc.asm"





.org 0x00
			rjmp reset

	; alle weiteren IRQs abfangen
.org INT0addr
			rjmp NMI_Handler
.org INT1addr
			rjmp RST_Handler
.org INT2addr
			rjmp IRQ_Handler
.org PCI0addr
			rjmp NMI_Handler
.org PCI1addr
			rjmp NMI_Handler
.org PCI2addr
			rjmp NMI_Handler
.org PCI3addr
			rjmp NMI_Handler
.org WDTaddr	
			rjmp NMI_Handler
.org OC2Aaddr
			rjmp NMI_Handler
.org OC2Baddr
			rjmp NMI_Handler
.org OVF2addr
			rjmp NMI_Handler
.org ICP1addr
			rjmp NMI_Handler
.org OC1Aaddr
			rjmp NMI_Handler
.org OC1Baddr
			rjmp NMI_Handler
.org OVF1addr
			rjmp NMI_Handler
.org OC0Aaddr
			rjmp NMI_Handler
.org OC0Baddr
			rjmp NMI_Handler
.org OVF0addr
			rjmp NMI_Handler
.org SPIaddr	
			rjmp NMI_Handler
.org URXC0addr
			rjmp NMI_Handler
.org UDRE0addr
			rjmp NMI_Handler
.org UTXC0addr
			rjmp NMI_Handler
.org ACIaddr	
			rjmp NMI_Handler
.org ADCCaddr
			rjmp NMI_Handler
.org ERDYaddr
			rjmp NMI_Handler
.org TWIaddr	
			rjmp NMI_Handler
.org SPMRaddr
			rjmp NMI_Handler


.org int_vectors_size
reset:
			ldi temp, low (RAMEND)
			out SPL, temp
			ldi temp, high(RAMEND)
			out SPH, temp

			clr zero
			ldi temp, 0x06
			mov ADC_helper1, temp
			swap temp
			mov ADC_helper2, temp

		; INT2 als IRQ konfigurieren:
			;ldi temp, 1<<INT2
			;out EIMSK, temp
		; IRQ enable:
			;sbi EIMSK, INT2



/*
		; Daten aus dem Flash ins SRAM kopieren fr 6510er-Software
			ldi ZL, low (RAM6510_content*2)
			ldi ZH, high(RAM6510_content*2)
			ldi YL, low (RAM6510_Start)
			ldi YH, high(RAM6510_Start)

		; Stackpointer auf 0x01FF setzen
			ser temp
			;ser temp2
			mov reg_SP, temp
		; IRQ auf active High fr Testzwecke
		; d.h. INT2 setzen, INT0 fr NMI Tests
			ldi temp, 1<<ISC20	; | 1<<ISC00 ; nur fr Test
			sts EICRA, temp
			;sbi EIMSK, INT0	; nur fr Test



			ldi loop, 0x08

		reset_0:
			; einfach 4094 Bytes, egal wieviele sinnvoll sind
			lpm temp2, Z+
			st Y+, temp2
			lpm temp2, Z+
			st Y+, temp2
			dec temp
			brne reset_0
			dec loop
			brne reset_0
*/
		; nops nur um fr den Simulator den CK auf festen 20er-Wert setzen
			nop
			nop	;hier  CK-Reset
			nop
			nop
			nop
			nop
			nop
			nop
			nop

			ldi ZL, low (Vector6510RST)			; Startadresse festlegen
			ldi ZH, high(Vector6510RST);hier fr Sim!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
			andi ZH, 0b00011111

			READ_RAMROM reg_PCL
			READ_RAMROM reg_PCH
			movw ZL, reg_PCL			;evtl. weg??

			;clr ZP_Highbyte
			ldi temp, ZPStart
			mov ZP_Highbyte, temp
MAIN:
			;****************************************
			; neu6.12.
			ldi temp, 10			; Sync-Zhler initialisieren
			mov SyncCount, temp
			ldi temp, (1<<CS10)		; Sync-Timer los
			sts TCCR1B, temp



			; eigentlich noch irgendeine Synchronisation oder sowas einfgen
			rjmp main_loop			; und los :-)








.include "OP_branches.asm"
.include "OP_Stack_inc.asm"
.include "OP_System_inc.asm"
.include "OP_LD_ST_inc.asm"

.include "OP_Logical_inc.asm"





			; Interrupts verwalten		7 Takte
			; ******************
	; erstmal alle SysIRQs vom AVR als NMI ausfhren, aktivieren ber jeweiliges IO
	;		bspw. USART-IRQ
	; das I-flag hat damit aber derzeit keine Bedeutung

			; Grundstzlich:
			;	PCL + PCH auf Stack
			;	Status auf Stack
			;	neuen Vektor holen (2 byte)
			;	Sprung auf neue Adresse

	;***************************************
	; Wichtig: Rcksprung nicht ber reti sondern rjmp, spart Takte
	; evtl. ZH vorladen mit Sprungtabelle


NMI_Handler:
	; hier 3 Takte vor ld (cli, READ_RDY)
			; erstes Problem: RDY msste whrend des IRQ abgefragt werden
			sbrc reg_Status2, flagB		; Aufruf von BRK aus??
			rjmp NMIBRK_Handler

			;sbr reg_Status2, flag_I
			;cbi IRQMask, IRQMPin		;2


			mov YL, reg_SP	;-------1Takt
			ldi YH, 0x01

			st Y, reg_PCH

			dec YL
			st Y, reg_PCL
			;RWAIT 5		; noch Status!!
			; Status ummodeln
			mov temp, reg_Status2
			mov temp2, reg_Status
			andi temp2, 0b00000011
			or temp2, temp
			bld reg_Status, 0x02
			bst temp2, 0x07
			bld reg_Status, 0x03
			bst temp2, 0x06
			; I-Flag aus IO auslesen???
			dec YL
			mov reg_SP, YL
			st Y, temp2		;4Takte
			dec reg_SP

			ldi ZL, low (Vector6510NMI)
			ldi ZH, high(Vector6510NMI)
			READ_RAMROM reg_PCL
			READ_RAMROM reg_PCH
			;ld reg_PCL, Y+
			;ld reg_PCH, Y+
			movw ZL, reg_PCL
			andi ZH, 0b00011111
			ldi loop, 7*MHz
			add SyncL, loop
			adc SyncH, zero

			pop temp;2
			pop temp			; AVR-Stack wieder frei machen, da kein reti
			rjmp main_loop_IRQ



IRQ_Handler:
; noch I-Flag setzen, IRQ abschalten
			sbrc reg_Status2, flagB		; Aufruf von BRK aus??
			rjmp IRQBRK_Handler

			sbr reg_Status2, flag_I
			cbi IRQMask, IRQMPin		;2

			mov YL, reg_SP	;-------1Takt
			ldi YH, 0x01
			;dec YL

			st Y, reg_PCH

			dec YL
			st Y, reg_PCL
			;RWAIT 5		; noch Status!!
			; Status ummodeln
			mov temp, reg_Status2
			mov temp2, reg_Status
			andi temp2, 0b00000011
			or temp2, temp
			bld reg_Status, 0x02
			bst temp2, 0x07
			bld reg_Status, 0x03
			bst temp2, 0x06
			; I-Flag aus IO auslesen???
			dec YL
			mov reg_SP, YL

			st Y, temp2		;4Takte

			dec reg_SP
			ldi ZL, low (Vector6510IRQ)
			ldi ZH, high(Vector6510IRQ)

			READ_RAMROM reg_PCL
			READ_RAMROM reg_PCH

			;ld reg_PCL, Y+
			;ld reg_PCH, Y+
			movw ZL, reg_PCL
			andi ZH, 0b00011111

			ldi loop, 7*MHz
			add SyncL, loop
			adc SyncH, zero

			pop temp
			pop temp			; AVR-Stack wieder frei machen, da kein reti
			rjmp main_loop_IRQ




RST_Handler:
	; hier 3 Takte vor ld (cli, READ_RDY)
			; erstes Problem: RDY msste whrend des IRQ abgefragt werden
			sbrc reg_Status2, flagB		; Aufruf von BRK aus??
			rjmp RSTBRK_Handler

			;sbr reg_Status2, flag_I
			;cbi IRQMask, IRQMPin		;2


			mov YL, reg_SP	;-------1Takt
			ldi YH, 0x01

			st Y, reg_PCH

			dec YL
			st Y, reg_PCL
			;RWAIT 5		; noch Status!!
			; Status ummodeln
			mov temp, reg_Status2
			mov temp2, reg_Status
			andi temp2, 0b00000011
			or temp2, temp
			bld reg_Status, 0x02
			bst temp2, 0x07
			bld reg_Status, 0x03
			bst temp2, 0x06
			; I-Flag aus IO auslesen???
			dec YL
			mov reg_SP, YL
			st Y, temp2		;4Takte
			dec reg_SP

			ldi ZL, low (Vector6510RST)
			ldi ZH, high(Vector6510RST)
			READ_RAMROM reg_PCL
			READ_RAMROM reg_PCH
			;ld reg_PCL, Y+
			;ld reg_PCH, Y+
			movw ZL, reg_PCL
			andi ZH, 0b00011111
			ldi loop, 7*MHz
			add SyncL, loop
			adc SyncH, zero

			pop temp;2
			pop temp			; AVR-Stack wieder frei machen, da kein reti
			rjmp main_loop_IRQ



IRQBRK_Handler:
		; PC und Status schon auf dem Stack
		; hier 0-Linie, ab hier 3 Takte

		; B-Flag wieder lschen???
			cbr reg_Status2, Flag_B
			ldi ZL, low (Vector6510IRQ)
			ldi ZH, high(Vector6510IRQ)
			READ_RAMROM reg_PCL
			READ_RAMROM reg_PCH
			;ld reg_PCL, Y+
			;ld reg_PCH, Y+
			ldi loop, 4*MHz
			add SyncL, loop
			adc SyncH, zero
			movw ZL, reg_PCL
			andi ZH, 0b00011111

			pop temp
			pop temp
			rjmp main_loop_IRQ


NMIBRK_Handler:
		; PC und Status schon auf dem Stack
			cbr reg_Status2, Flag_B
			ldi ZL, low (Vector6510NMI)
			ldi ZH, high(Vector6510NMI)
			READ_RAMROM reg_PCL
			READ_RAMROM reg_PCH
			;ld reg_PCL, Y+
			;ld reg_PCH, Y+
			ldi loop, 4*MHz
			add SyncL, loop
			adc SyncH, zero
			movw ZL, reg_PCL
			andi ZH, 0b00011111

			pop temp;2
			pop temp
			rjmp main_loop_IRQ

RSTBRK_Handler:
		; PC und Status schon auf dem Stack
			cbr reg_Status2, Flag_B
			ldi ZL, low (Vector6510RST)
			ldi ZH, high(Vector6510RST)
			READ_RAMROM reg_PCL
			READ_RAMROM reg_PCH
			;ld reg_PCL, Y+
			;ld reg_PCH, Y+
			ldi loop, 4*MHz
			add SyncL, loop
			adc SyncH, zero
			movw ZL, reg_PCL
			andi ZH, 0b00011111

			pop temp;2
			pop temp
			rjmp main_loop_IRQ



main_loop:
			sei
			;movw ZL, reg_PCL			;evtl. weg??
			;nop
			andi ZH, 0b00011111
main_loop_IRQ:
			cli
			; *************************
			; opcode lesen
			;READ_RDY
			dec SyncCount
			brne main_weiter
			rcall Do_Sync
		main_weiter:
			cpi ZH, ROMBASE	; test, ob RAM oder ROM
			brsh weiter
			ld opcode, Z+	
			rjmp weiter2	;6
		weiter:
			lpm opcode, Z+	;6
		weiter2:


			movw reg_PCL, ZL
			mov ZL, opcode
			ldi ZH, high(opcodes_table)
			ijmp	;16 inkl.rjmp



;main_loop:
;			sei
;			nop
;main_loop_IRQ:
;			cli
;			; *************************
;			; opcode lesen
;			READ_RDY
;			ld opcode, X+
;
;opcode_switch:
;			ldi ZL, low (opcodes_table)
;			ldi ZH, high(opcodes_table)
;			nop
;			add ZL, opcode
;			adc ZH, zero
;			nop
;			nop

;			ijmp			; +rjmp = 4 Takte



Warteschleife:
; wenn RDY zeigt, dass Leitung nicht frei
; hier warten, bis wieder frei
	; sbic RDY
	; rcall 4
	; READ
			RWAIT 7		;13
			sbis SYS_PORT, RDY_bit
			ret		;5
			RWAIT 17		;7
			rjmp Warteschleife
		; Fr andere als m644: ret 1 Takt weniger!!!!!!!!!!!!!!!!!!!!!



; wie knnte eine Sync-Funktion aussehen, wenn die Genauigkeit bspw. nur alle 10 Takte ntig wre??
;
;	Genauigkeit auf 16-bit bzgl. Warteschleife, Restwert mitnehmen, damit das Gesamtmittel bleibt
; Es sollten Maximal 2.400 Tics abgelaufen sein, also tH=0b0000xxxx
		; Timer1 (16-bit) luft,
		;	reg 24/25 als Zhler????
; Probleme, wenn's knapp wird
;	ich muss noch irgendwie unterscheiden, ob berlauf mit oder ohne Sync passiert
Do_Sync:
			ldi temp, 10
			mov SyncCount, temp
			adiw SyncL, 31		; Takte fr dieses hier(maximal)
			lds temp, TCNT1L
			lds temp2, TCNT1H
			;sub temp, SyncL
			;sbc temp2, SyncH
			sub SyncL, temp
			sbc SyncH, temp2
			brcs Sync_overflow

			;mov SyncL, temp		;rest behalten
			mov temp, SyncL
			;mov temp2, SyncH
			swap SyncH
			swap temp
			andi temp, 0b00001111
			or temp, SyncH
			breq Sync_nowait

			nop				;ausgleich fr den fehlenden 16.Takt
		Sync_warten:
			WAIT13
			dec temp
			brne Sync_warten

		Sync_nowait:
			; Rest merken, timer auf 0
			clr SyncH
			andi SyncL, 0b00001111
			sts TCNT1H, zero
			sts TCNT1L, zero		;22+ plus rcall+ret=31+
			ret

		Sync_overflow:
			;zu langsam: timer vorziehen
			;sbiw temp, 8			;eingesparte Takte wieder weg
			;subi temp, 8
			; temp+temp2 -> negativer Rest
			; sollte mgl. nicht ber 255 gehen
			;neg temp
			;subi temp, 10			; Vorsicht, wenn'S unter 10 Takte Abweichung waren
									; evtl. Pause einfgen und die Zeit immer 'verbraten'???
			neg SyncL
			sbiw SyncL, 10
			sts TCNT1H, zero
			sts TCNT1L, SyncL		;14 Takte plus rcall+ret=23
			ret






.include "OPcodeTabelle.asm"
.include "RW_switch.asm"

.include "OP_IN_DE_inc.asm"
.include "OP_CP_inc.asm"
.include "OP_Shift_inc.asm"
.include "OP_illegal.asm"
.include "OP_ADC_SBC_inc.asm"
.include "IO_inc.asm"

.include "6510CodeSeg_inc.asm"
.include "SRAM_inc.asm"
.include "notizen_inc.asm"

; ***************************************************************
; schonmal testweise das KIM-1 ROM:
; ***************************************************************

.include "k1rom_inc.asm"























