;*********************************************************************************************************
;		Zyklengenaue Emulation eines MOS6510 auf einem ATmega
;			(c) Mark Leyer 2010
;*********************************************************************************************************




/*
opcodes_table:
.db low(opcode_BRK), high(opcode_BRK), low(opcode_ORA_InX), high(opcode_ORA_InX)		;0x00 0x01
.db low(opcode_INVAL), high(opcode_INVAL), low(opcode_SLO), high(opcode_SLO)			;0x02 0x03
.db low(opcode_NOP), high(opcode_NOP), low(opcode_ORA_ZP), high(opcode_ORA_ZP)			;0x04 0x05
.db low(opcode_ASL_ZP), high(opcode_ASL_ZP), low(opcode_SLO), high(opcode_SLO)			;0x06 0x07
.db low(opcode_PHP), high(opcode_PHP), low(opcode_ORA_immed), high(opcode_ORA_immed)	;0x08 0x09
.db low(opcode_ASL_accu), high(opcode_ASL_accu), low(opcode_ANC), high(opcode_ANC)		;0x0A 0x0B
.db low(opcode_NOP), high(opcode_NOP), low(opcode_ORA_Abs), high(opcode_ORA_Abs)		;0x0C 0x0D
.db low(opcode_ASL_Abs), high(opcode_ASL_Abs), low(opcode_SLO), high(opcode_SLO)		;0x0E 0x0F

.db low(opcode_BPL), high(opcode_BPL), low(opcode_ORA_InY), high(opcode_ORA_InY)		;0x10 0x11
.db low(opcode_INVAL), high(opcode_INVAL), low(opcode_SLO), high(opcode_SLO)			;0x12 0x13
.db low(opcode_NOP), high(opcode_NOP), low(opcode_ORA_ZPX), high(opcode_ORA_ZPX)		;0x14 0x15
.db low(opcode_ASL_ZPX), high(opcode_ASL_ZPX), low(opcode_SLO), high(opcode_SLO)		;0x16 0x17
.db low(opcode_CLC), high(opcode_CLC), low(opcode_ORA_Abs_Y), high(opcode_ORA_Abs_Y)	;0x18 0x19
.db low(opcode_NOP), high(opcode_NOP), low(opcode_SLO), high(opcode_SLO)				;0x1A 0x1B
.db low(opcode_NOP), high(opcode_NOP), low(opcode_ORA_Abs_X), high(opcode_ORA_Abs_X)	;0x1C 0x1D
.db low(opcode_ASL_Abs_X), high(opcode_ASL_Abs_X), low(opcode_SLO), high(opcode_SLO)	;0x1E 0x1F

.db low(opcode_JSR), high(opcode_JSR), low(opcode_AND_InX), high(opcode_AND_InX)		;0x20 0x21
.db low(opcode_INVAL), high(opcode_INVAL), low(opcode_RLA), high(opcode_RLA)			;0x22 0x23
.db low(opcode_BIT), high(opcode_BIT), low(opcode_AND_ZP), high(opcode_AND_ZP)			;0x24 0x25
.db low(opcode_ROL_ZP), high(opcode_ROL_ZP), low(opcode_RLA), high(opcode_RLA)			;0x26 0x27
.db low(opcode_PLP), high(opcode_PLP), low(opcode_AND_immed), high(opcode_AND_immed)	;0x28 0x29
.db low(opcode_ROL_accu), high(opcode_ROL_accu), low(opcode_ANC), high(opcode_ANC)		;0x2A 0x2B
.db low(opcode_BIT), high(opcode_BIT), low(opcode_AND_Abs), high(opcode_AND_Abs)		;0x2C 0x2D
.db low(opcode_ROL_Abs), high(opcode_ROL_Abs), low(opcode_RLA), high(opcode_RLA)		;0x2E 0x2F

.db low(opcode_BMI), high(opcode_BMI), low(opcode_AND_InY), high(opcode_AND_InY)		;0x30 0x31
.db low(opcode_INVAL), high(opcode_INVAL), low(opcode_RLA), high(opcode_RLA)			;0x32 0x33
.db low(opcode_NOP), high(opcode_NOP), low(opcode_AND_ZPX), high(opcode_AND_ZPX)		;0x34 0x35
.db low(opcode_ROL_ZPX), high(opcode_ROL_ZPX), low(opcode_RLA), high(opcode_RLA)		;0x36 0x37
.db low(opcode_SEC), high(opcode_SEC), low(opcode_AND_Abs_Y), high(opcode_AND_Abs_Y)	;0x38 0x39
.db low(opcode_NOP), high(opcode_NOP), low(opcode_RLA), high(opcode_RLA)				;0x3A 0x3B
.db low(opcode_NOP), high(opcode_NOP), low(opcode_AND_Abs_X), high(opcode_AND_Abs_X)	;0x3C 0x3D
.db low(opcode_ROL_Abs_X), high(opcode_ROL_Abs_X), low(opcode_RLA), high(opcode_RLA)	;0x3E 0x3F

.db low(opcode_RTI), high(opcode_RTI), low(opcode_EOR_InX), high(opcode_EOR_InX)		;0x40 0x41
.db low(opcode_INVAL), high(opcode_INVAL), low(opcode_SRE	), high(opcode_SRE)			;0x42 0x43
.db low(opcode_NOP), high(opcode_NOP), low(opcode_EOR_ZP), high(opcode_EOR_ZP)			;0x44 0x45
.db low(opcode_LSR_ZP), high(opcode_LSR_ZP), low(opcode_SRE), high(opcode_SRE)			;0x46 0x47
.db low(opcode_PHA), high(opcode_PHA), low(opcode_EOR_immed), high(opcode_EOR_immed)	;0x48 0x49
.db low(opcode_LSR_accu), high(opcode_LSR_accu), low(opcode_ASR), high(opcode_ASR)		;0x4A 0x4B
.db low(opcode_JMP), high(opcode_JMP), low(opcode_EOR_Abs), high(opcode_EOR_Abs)		;0x4C 0x4D
.db low(opcode_LSR_Abs), high(opcode_LSR_Abs), low(opcode_SRE), high(opcode_SRE)		;0x4E 0x4F

.db low(opcode_BVC), high(opcode_BVC), low(opcode_EOR_InY), high(opcode_EOR_InY)		;0x50 0x51
.db low(opcode_INVAL), high(opcode_INVAL), low(opcode_SRE), high(opcode_SRE)			;0x52 0x53
.db low(opcode_NOP), high(opcode_NOP), low(opcode_EOR_ZPX), high(opcode_EOR_ZPX)		;0x54 0x55
.db low(opcode_LSR_ZPX), high(opcode_LSR_ZPX), low(opcode_SRE), high(opcode_SRE)		;0x56 0x57
.db low(opcode_CLI), high(opcode_CLI), low(opcode_EOR_Abs_Y), high(opcode_EOR_Abs_Y)	;0x58 0x59
.db low(opcode_NOP), high(opcode_NOP), low(opcode_SRE), high(opcode_SRE)				;0x5A 0x5B
.db low(opcode_NOP), high(opcode_NOP), low(opcode_EOR_Abs_X), high(opcode_EOR_Abs_X)	;0x5C 0x5D
.db low(opcode_LSR_Abs_X), high(opcode_LSR_Abs_X), low(opcode_SRE), high(opcode_SRE)	;0x5E 0x5F

.db low(opcode_RTS), high(opcode_RTS), low(opcode_ADC_InX), high(opcode_ADC_InX)		;0x60 0x61
.db low(opcode_INVAL), high(opcode_INVAL), low(opcode_RRA), high(opcode_RRA)			;0x62 0x63
.db low(opcode_NOP), high(opcode_NOP), low(opcode_ADC_ZP), high(opcode_ADC_ZP)			;0x64 0x65
.db low(opcode_ROR_ZP), high(opcode_ROR_ZP), low(opcode_RRA), high(opcode_RRA)			;0x66 0x67
.db low(opcode_PLA), high(opcode_PLA), low(opcode_ADC_immed), high(opcode_ADC_immed)	;0x68 0x69
.db low(opcode_ROR_accu), high(opcode_ROR_accu), low(opcode_ARR), high(opcode_ARR)		;0x6A 0x6B
.db low(opcode_JMP), high(opcode_JMP), low(opcode_ADC_Abs), high(opcode_ADC_Abs)		;0x6C 0x6D
.db low(opcode_ROR_Abs), high(opcode_ROR_Abs), low(opcode_RRA), high(opcode_RRA)		;0x6E 0x6F

.db low(opcode_BVS), high(opcode_BVS), low(opcode_ADC_InY), high(opcode_ADC_InY)		;0x70 0x71
.db low(opcode_INVAL), high(opcode_INVAL), low(opcode_RRA), high(opcode_RRA)			;0x72 0x73
.db low(opcode_NOP), high(opcode_NOP), low(opcode_ADC_ZPX), high(opcode_ADC_ZPX)		;0x74 0x75
.db low(opcode_ROR_ZP), high(opcode_ROR_ZP), low(opcode_RRA), high(opcode_RRA)			;0x76 0x77
.db low(opcode_SEI), high(opcode_SEI), low(opcode_ADC_Abs_Y), high(opcode_ADC_Abs_Y)	;0x78 0x79
.db low(opcode_NOP), high(opcode_NOP), low(opcode_RRA), high(opcode_RRA)				;0x7A 0x7B
.db low(opcode_NOP), high(opcode_NOP), low(opcode_ADC_Abs_X), high(opcode_ADC_Abs_X)	;0x7C 0x7D
.db low(opcode_ROR_Abs_X), high(opcode_ROR_Abs_X), low(opcode_RRA), high(opcode_RRA)	;0x7E 0x7F

.db low(opcode_NOP), high(opcode_NOP), low(opcode_STA_InX), high(opcode_STA_InX)		;0x80
.db low(opcode_NOP), high(opcode_NOP), low(opcode_SAX), high(opcode_SAX)
.db low(opcode_STY_ZP), high(opcode_STY_ZP), low(opcode_STA_ZP), high(opcode_STA_ZP)
.db low(opcode_STX_ZP), high(opcode_STX_ZP), low(opcode_SAX), high(opcode_SAX)
.db low(opcode_DEY), high( opcode_DEY), low(opcode_NOP), high(opcode_NOP)
.db low(opcode_TXA), high(opcode_TXA), low(opcode_ANE), high(opcode_ANE)
.db low(opcode_STY_Abs), high(opcode_STY_Abs), low(opcode_STA_Abs), high(opcode_STA_Abs )
.db low(opcode_STX_Abs), high(opcode_STX_Abs), low(opcode_SAX), high(opcode_SAX)

.db low(opcode_BCC), high(opcode_BCC), low(opcode_STA_InY), high(opcode_STA_InY)		;0x90
.db low(opcode_INVAL), high(opcode_INVAL), low(opcode_SHA), high(opcode_SHA)
.db low(opcode_STY_ZPX), high(opcode_STY_ZPX), low(opcode_STA_ZPX), high(opcode_STA_ZPX)
.db low(opcode_STX_ZPY), high(opcode_STX_ZPY), low(opcode_SAX), high(opcode_SAX)
.db low(opcode_TYA), high(opcode_TYA), low(opcode_STA_Abs_Y), high(opcode_STA_Abs_Y)
.db low(opcode_TXS), high(opcode_TXS), low(opcode_SHS), high(opcode_SHS)
.db low(opcode_SHY), high(opcode_SHY), low(opcode_STA_Abs_X), high(opcode_STA_Abs_X)
.db low(opcode_SHX), high(opcode_SHX), low(opcode_SHA), high(opcode_SHA)


.db low(opcode_LDY_immed), high(opcode_LDY_immed), low(opcode_LDA_InX), high(opcode_LDA_InX)	;0xA0
.db low(opcode_LDX_immed), high(opcode_LDX_immed), low(opcode_LAX), high(opcode_LAX)
.db low(opcode_LDY_ZP), high(opcode_LDY_ZP), low(opcode_LDA_ZP), high(opcode_LDA_ZP)
.db low(opcode_LDX_ZP), high(opcode_LDX_ZP), low(opcode_LAX), high(opcode_LAX)
.db low(opcode_TAY), high(opcode_TAY), low(opcode_LDA_immed), high(opcode_LDA_immed)
.db low(opcode_TAX), high(opcode_TAX), low(opcode_LXA), high(opcode_LXA)
.db low(opcode_LDY_Abs), high(opcode_LDY_Abs), low(opcode_LDA_Abs), high(opcode_LDA_Abs)
.db low(opcode_LDX_Abs), high(opcode_LDX_Abs), low(opcode_LAX), high(opcode_LAX)

.db low(opcode_BCS), high(opcode_BCS), low(opcode_LDA), high(opcode_LDA)			;0xB0
.db low(opcode_INVAL), high(opcode_INVAL), low(opcode_LAX), high(opcode_LAX)
.db low(opcode_LDY_ZPX), high(opcode_LDY_ZPX), low(opcode_LDA_ZPX), high(opcode_LDA_ZPX)
.db low(opcode_LDX_ZPY), high(opcode_LDX_ZPY), low(opcode_LAX), high(opcode_LAX)
.db low(opcode_CLV), high(opcode_CLV), low(opcode_LDA_Abs_Y), high(opcode_LDA_Abs_Y)
.db low(opcode_TSX), high(opcode_TSX), low(opcode_LAS), high(opcode_LAS)
.db low(opcode_LDY_Abs_X), high(opcode_LDY_Abs_X), low(opcode_LDA_Abs_X), high(opcode_LDA_Abs_X)
.db low(opcode_LDX_Abs_Y), high(opcode_LDX_Abs_Y), low(opcode_LAX), high(opcode_LAX)

.db low(opcode_CPY_immed), high(opcode_CPY_immed), low(opcode_CMP_InX), high(opcode_CMP_InX)	;0xC0
.db low(opcode_NOP), high(opcode_NOP), low(opcode_DCP), high(opcode_DCP)
.db low(opcode_CPY_ZP), high(opcode_CPY_ZP), low(opcode_CMP_ZP), high(opcode_CMP_ZP)
.db low(opcode_DEC_ZP), high(opcode_DEC_ZP), low(opcode_DCP), high(opcode_DCP)
.db low(opcode_DCP), high(opcode_DCP), low(opcode_CMP_immed), high(opcode_CMP_immed)
.db low(opcode_DEX), high(opcode_DEX), low(opcode_SBX), high(opcode_SBX)
.db low(opcode_CPY_Abs), high(opcode_CPY_Abs), low(opcode_CMP_Abs), high(opcode_CMP_Abs)
.db low(opcode_DEC_Abs), high(opcode_DEC_Abs), low(opcode_DCP), high(opcode_DCP)

.db low(opcode_BNE), high(opcode_BNE), low(opcode_CMP_InY), high(opcode_CMP_InY)		;0xD0
.db low(opcode_INVAL), high(opcode_INVAL), low(opcode_DCP), high(opcode_DCP)
.db low(opcode_NOP), high(opcode_NOP), low(opcode_CMP_ZPX), high(opcode_CMP_ZPX)
.db low(opcode_DEC_ZPX), high(opcode_DEC_ZPX), low(opcode_DCP), high(opcode_DCP)
.db low(opcode_CLD), high(opcode_CLD), low(opcode_CMP_Abs_Y), high(opcode_CMP_Abs_Y)
.db low(opcode_NOP), high(opcode_NOP), low(opcode_DCP), high(opcode_DCP)
.db low(opcode_NOP), high(opcode_NOP), low(opcode_CMP_Abs_X), high(opcode_CMP_Abs_X)
.db low(opcode_DEC_Abs_X), high(opcode_DEC_Abs_X), low(opcode_DCP), high(opcode_DCP)


.db low(opcode_CPX_immed), high(opcode_CPX_immed), low(opcode_SBC_InX), high(opcode_SBC_InX)	;0xE0
.db low(opcode_NOP), high(opcode_NOP), low(opcode_ISB), high(opcode_ISB)
.db low(opcode_CPX_ZP), high(opcode_CPX_ZP), low(opcode_SBC_ZP), high(opcode_SBC_ZP)
.db low(opcode_INC_ZP), high(opcode_INC_ZP), low(opcode_ISB), high(opcode_ISB)
.db low(opcode_INX), high(opcode_INX), low(opcode_SBC_immed), high(opcode_SBC_immed)
.db low(opcode_NOP), high(opcode_NOP), low(opcode_SBC), high(opcode_SBC)
.db low(opcode_CPX_Abs), high(opcode_CPX_Abs), low(opcode_SBC_Abs), high(opcode_SBC_Abs)
.db low(opcode_INC_Abs), high(opcode_INC_Abs), low(opcode_ISB), high(opcode_ISB)

.db low(opcode_BEQ), high(opcode_BEQ), low(opcode_SBC_InY), high(opcode_SBC_InY)		;0xF0
.db low(opcode_INVAL), high(opcode_INVAL), low(opcode_ISB), high(opcode_ISB)
.db low(opcode_NOP), high(opcode_NOP), low(opcode_SBC_ZPX), high(opcode_SBC_ZPX)
.db low(opcode_INC_ZPX), high(opcode_INC_ZPX), low(opcode_ISB), high(opcode_ISB)
.db low(opcode_ISB), high(opcode_ISB), low(opcode_SBC_Abs_Y), high(opcode_SBC_Abs_Y)
.db low(opcode_NOP), high(opcode_NOP), low(opcode_ISB), high(opcode_ISB)
.db low(opcode_NOP), high(opcode_NOP), low(opcode_SBC_Abs_X), high(opcode_SBC_Abs_X)
.db low(opcode_INC_Abs_X), high(opcode_INC_Abs_X), low(opcode_ISB), high(opcode_ISB)
*/




.org 0x0500				; muss lowbyte =0 sein
						; evtl. anpasse via map-file
opcodes_table:
rjmp opcode_BRK			;0x00
rjmp opcode_ORA_InX		;			erl.
rjmp opcode_INVAL		;			erl.
rjmp opcode_SLO			;			erl.
rjmp opcode_NOP			;			erl.
rjmp opcode_ORA_ZP		;			erl.
rjmp opcode_ASL_ZP		;			erl.
rjmp opcode_SLO			;			erl.
rjmp opcode_PHP			;			erl.
rjmp opcode_ORA_immed	;			erl.
rjmp opcode_ASL_accu	;			erl.
rjmp opcode_SZP_immed	;			erl.		********************* neuer Opcode!!!!
rjmp opcode_NOP			;			erl.
rjmp opcode_ORA_Abs		;			erl.
rjmp opcode_ASL_Abs		;			erl.
rjmp opcode_SLO			;0x0F		erl.

rjmp opcode_BPL			;0x10		erl.
rjmp opcode_ORA_InY		;			erl.
rjmp opcode_INVAL		;			erl.
rjmp opcode_SLO			;			erl.
rjmp opcode_NOP			;			erl.
rjmp opcode_ORA_ZPX		;			erl.
rjmp opcode_ASL_ZPX		;			erl.
rjmp opcode_SLO			;			erl.
rjmp opcode_CLC			;			erl.
rjmp opcode_ORA_Abs_Y	;			erl.
rjmp opcode_NOP			;			erl.
rjmp opcode_SLO			;			erl.
rjmp opcode_NOP			;			erl.
rjmp opcode_ORA_Abs_X	;			erl.
rjmp opcode_ASL_Abs_X	;			erl.
rjmp opcode_SLO			;0x1F		erl.

rjmp opcode_JSR			;0x20		erl.
rjmp opcode_AND_InX		;			erl.
rjmp opcode_INVAL		;			erl.
rjmp opcode_RLA			;			erl.
rjmp opcode_BIT_ZP		;			erl.
rjmp opcode_AND_ZP		;			erl.
rjmp opcode_ROL_ZP		;			erl.
rjmp opcode_RLA			;			erl.
rjmp opcode_PLP			;			erl.
rjmp opcode_AND_immed	;			erl.
rjmp opcode_ROL_accu	;			erl.
rjmp opcode_ANC			;			erl.
rjmp opcode_BIT_Abs		;			erl.
rjmp opcode_AND_Abs		;			erl.
rjmp opcode_ROL_Abs		;			erl.
rjmp opcode_RLA			;0x2F		erl.

rjmp opcode_BMI			;0x30		erl.
rjmp opcode_AND_InY		;			erl.
rjmp opcode_INVAL		;			erl.
rjmp opcode_RLA			;			erl.
rjmp opcode_NOP			;			erl.
rjmp opcode_AND_ZPX		;			erl.
rjmp opcode_ROL_ZPX		;			erl.
rjmp opcode_RLA			;			erl.
rjmp opcode_SEC			;			erl.
rjmp opcode_AND_Abs_Y	;			erl.
rjmp opcode_NOP			;			erl.
rjmp opcode_RLA			;			erl.
rjmp opcode_NOP			;			erl.
rjmp opcode_AND_Abs_X	;			erl.
rjmp opcode_ROL_Abs_X	;			erl.
rjmp opcode_RLA			;0x3F		erl.

rjmp opcode_RTI			;0x40
rjmp opcode_EOR_InX		;			erl.
rjmp opcode_INVAL		;			erl.
rjmp opcode_SRE			;			erl.
rjmp opcode_NOP			;			erl.
rjmp opcode_EOR_ZP		;			erl.
rjmp opcode_LSR_ZP		;			erl.
rjmp opcode_SRE			;			erl.
rjmp opcode_PHA			;			erl.
rjmp opcode_EOR_immed	;			erl.
rjmp opcode_LSR_accu	;			erl.
rjmp opcode_ASR			;			erl.
rjmp opcode_JMP			;			erl.
rjmp opcode_EOR_Abs		;			erl.
rjmp opcode_LSR_Abs		;			erl.
rjmp opcode_SRE			;0x4F		erl.

rjmp opcode_BVC			;0x50		erl.
rjmp opcode_EOR_InY		;			erl.
rjmp opcode_INVAL		;			erl.
rjmp opcode_SRE			;			erl.
rjmp opcode_NOP			;			erl.
rjmp opcode_EOR_ZPX		;			erl.
rjmp opcode_LSR_ZPX		;			erl.
rjmp opcode_SRE			;			erl.
rjmp opcode_CLI			;			erl.
rjmp opcode_EOR_Abs_Y	;			erl.
rjmp opcode_NOP			;			erl.
rjmp opcode_SRE			;			erl.
rjmp opcode_NOP			;			erl.
rjmp opcode_EOR_Abs_X	;			erl.
rjmp opcode_LSR_Abs_X	;			erl.
rjmp opcode_SRE			;0x5F		erl.

rjmp opcode_RTS			;0x60		erl.
rjmp opcode_ADC_InX		;			erl.
rjmp opcode_INVAL		;			erl.
rjmp opcode_RRA			;			erl.
rjmp opcode_NOP			;			erl.
rjmp opcode_ADC_ZP		;			erl.
rjmp opcode_ROR_ZP		;			erl.
rjmp opcode_RRA			;			erl.
rjmp opcode_PLA			;			erl.
rjmp opcode_ADC_immed	;			erl.
rjmp opcode_ROR_accu	;			erl.
rjmp opcode_ARR			;			erl.
rjmp opcode_JMP_indirect;			erl.
rjmp opcode_ADC_Abs		;			erl.
rjmp opcode_ROR_Abs		;			erl.
rjmp opcode_RRA			;0x6F		erl.

rjmp opcode_BVS			;0x70		erl.
rjmp opcode_ADC_InY		;			erl.
rjmp opcode_INVAL		;			erl.
rjmp opcode_RRA			;			erl.
rjmp opcode_NOP			;			erl.
rjmp opcode_ADC_ZPX		;			erl.
rjmp opcode_ROR_ZPX		;			erl.
rjmp opcode_RRA			;			erl.
rjmp opcode_SEI			;			erl.
rjmp opcode_ADC_Abs_Y	;			erl.
rjmp opcode_NOP			;			erl.
rjmp opcode_RRA			;			erl.
rjmp opcode_NOP			;			erl.
rjmp opcode_ADC_Abs_X	;			erl.
rjmp opcode_ROR_Abs_X	;			erl.
rjmp opcode_RRA			;0x7F		erl.

rjmp opcode_NOP			;0x80		erl. 
rjmp opcode_STA_InX		;			erl.
rjmp opcode_NOP			;			erl.
rjmp opcode_SAX			;			erl.
rjmp opcode_STY_ZP		;			erl.
rjmp opcode_STA_ZP 		;			erl. 
rjmp opcode_STX_ZP		;			erl.
rjmp opcode_SAX			;			erl.
rjmp opcode_DEY			;			erl.  
rjmp opcode_NOP			;			erl. 
rjmp opcode_TXA  		;			erl.
rjmp opcode_ANE			;			erl.
rjmp opcode_STY_Abs		;			erl.  
rjmp opcode_STA_Abs		;			erl.  
rjmp opcode_STX_Abs 	;			erl. 
rjmp opcode_SAX			;0x8F 		erl.

rjmp opcode_BCC			;0x90		erl.
rjmp opcode_STA_InY		;			erl.
rjmp opcode_INVAL		;			erl.
rjmp opcode_SHA			;			erl.
rjmp opcode_STY_ZPX		;			erl.
rjmp opcode_STA_ZPX		;			erl.
rjmp opcode_STX_ZPY		;			erl.
rjmp opcode_SAX			;			erl.
rjmp opcode_TYA			;			erl.
rjmp opcode_STA_Abs_Y	;			erl.
rjmp opcode_TXS			;			erl.
rjmp opcode_SHS			;			erl.
rjmp opcode_SHY			;			erl.
rjmp opcode_STA_Abs_X	;			erl.
rjmp opcode_SHX			;			erl.
rjmp opcode_SHA			;			erl.

rjmp opcode_LDY_immed	;0xA0		erl.
rjmp opcode_LDA_InX		;			erl.
rjmp opcode_LDX_immed	;			erl.
rjmp opcode_LAX			;			erl.
rjmp opcode_LDY_ZP		;			erl.
rjmp opcode_LDA_ZP		;			erl.
rjmp opcode_LDX_ZP		;			erl.
rjmp opcode_LAX			;			erl.
rjmp opcode_TAY			;			erl.
rjmp opcode_LDA_immed	;			erl.
rjmp opcode_TAX			;			erl.
rjmp opcode_LXA			;			erl.
rjmp opcode_LDY_Abs		;			erl.
rjmp opcode_LDA_Abs		;			erl.
rjmp opcode_LDX_Abs		;			erl.
rjmp opcode_LAX			;0xAF		erl.

rjmp opcode_BCS			;0xB0		erl.
rjmp opcode_LDA_InY		;			erl.
rjmp opcode_INVAL		;			erl.
rjmp opcode_LAX			;			erl.
rjmp opcode_LDY_ZPX		;			erl.
rjmp opcode_LDA_ZPX		;			erl.
rjmp opcode_LDX_ZPY		;			erl.
rjmp opcode_LAX			;			erl.
rjmp opcode_CLV			;			erl.
rjmp opcode_LDA_Abs_Y	;			erl.
rjmp opcode_TSX			;			erl.
rjmp opcode_LAS			;			erl.
rjmp opcode_LDY_Abs_X	;			erl.
rjmp opcode_LDA_Abs_X	;			erl.
rjmp opcode_LDX_Abs_Y	;			erl.
rjmp opcode_LAX			;0xBF		erl.

rjmp opcode_CPY_immed	;0xC0		erl.
rjmp opcode_CMP_InX		;			erl.
rjmp opcode_NOP			;			erl.
rjmp opcode_DCP			;			erl.
rjmp opcode_CPY_ZP		;			erl.
rjmp opcode_CMP_ZP		;			erl.
rjmp opcode_DEC_ZP		;			erl.
rjmp opcode_DCP			;			erl.
rjmp opcode_INY			;			erl.
rjmp opcode_CMP_immed	;			erl.
rjmp opcode_DEX			;			erl.
rjmp opcode_SBX			;			erl.
rjmp opcode_CPY_Abs		;			erl.
rjmp opcode_CMP_Abs		;			erl.
rjmp opcode_DEC_Abs		;			erl.
rjmp opcode_DCP			;0xCF		erl.

rjmp opcode_BNE			;0xD0		erl.
rjmp opcode_CMP_InY		;			erl.
rjmp opcode_INVAL		;			erl.
rjmp opcode_DCP			;			erl.
rjmp opcode_NOP			;			erl.
rjmp opcode_CMP_ZPX		;			erl.
rjmp opcode_DEC_ZPX		;			erl.
rjmp opcode_DCP			;			erl.
rjmp opcode_CLD			;			erl.
rjmp opcode_CMP_Abs_Y	;			erl.
rjmp opcode_NOP			;			erl.
rjmp opcode_DCP			;			erl.
rjmp opcode_NOP			;			erl.
rjmp opcode_CMP_Abs_X	;			erl.
rjmp opcode_DEC_Abs_X	;			erl.
rjmp opcode_DCP			;0xDF		erl.

rjmp opcode_CPX_immed	;0xE0		erl.
rjmp opcode_SBC_InX		;			erl.
rjmp opcode_NOP			;			erl.
rjmp opcode_ISB			;			erl.
rjmp opcode_CPX_ZP		;			erl.
rjmp opcode_SBC_ZP		;			erl.
rjmp opcode_INC_ZP		;			erl.
rjmp opcode_ISB			;			erl.
rjmp opcode_INX			;			erl.
rjmp opcode_SBC_immed	;			erl.
rjmp opcode_NOP			;			erl.
rjmp opcode_SBC_immed	;	???		erl.
rjmp opcode_CPX_Abs		;			erl.
rjmp opcode_SBC_Abs		;			erl.
rjmp opcode_INC_Abs		;			erl.
rjmp opcode_ISB			;			erl.

rjmp opcode_BEQ			;0xF0		erl.
rjmp opcode_SBC_InY		;			erl.
rjmp opcode_INVAL		;			erl.
rjmp opcode_ISB			;			erl.
rjmp opcode_NOP			;			erl.
rjmp opcode_SBC_ZPX		;			erl.
rjmp opcode_INC_ZPX		;			erl.
rjmp opcode_ISB			;			erl.
rjmp opcode_SED			;			erl.
rjmp opcode_SBC_Abs_Y	;			erl.
rjmp opcode_NOP			;			erl.
rjmp opcode_ISB			;			erl.
rjmp opcode_NOP			;			erl.
rjmp opcode_SBC_Abs_X	;			erl.
rjmp opcode_INC_Abs_X	;			erl.
rjmp opcode_ISB			;0xFF		erl.















